;******************************************************************************
; file:      intdiv.asm
; content:   sf32b integer divide routine, 32-bit/32-bit
; author:    Martin Raubuch
; created:   21Oct2011
; modified:  21Oct2011
; Property of RACORS GmbH, Ginsterweg 1a, 85598 Baldham, Germany
;******************************************************************************
; ABI Conventions, allocation order
; RZ           used as stack/frame pointer
; R0-R3        data argument passing, may be destroyed by called function
; RF-RC        data/pointer arg. passing, may be destroyed by called function
; RW,RX,RY     data/pointer arg. passing, may be destroyed by called function
; R0,R1        return value
; R4-RB/RP-RV  if modified must be saved/restored by called function	
;******************************************************************************

;***************************** Integer Divide *********************************
; Performs an unsigned integer divide of the specified dividend and divisor
; arg R0 = dividend
; arg R1 = divisor
; returns R0 = remainder, R1 = quotient
;******************************************************************************
IntDiv3232:
   clzr   R0,R2			; divident size
   clzr   R1,R3			; divisor size
   subf   R2,R3			; size difference
   move   0,R1,ICR		; if divisor size > divident size quotient = 0
   brnc   1f			; if not continue at 1f
   rtsr				; if yes quit
1: shlz   R3,R1,R2 	        ; R2 = aligned divisor
   addt   1,R3  		; iteration count
   move   0,R1			; initialize quotient to 0
   move   R3,LC 		; LC = iteration count
1: comp   R2,R0			; divisor <= divident ?
   shlz   1,R1			; quotient <<= 1
   btst   0,R1,INC		; if yes set bit 0 of quotient
   subf   R2,R0,INC		; if yes divident -= dividsor
   shlz   1,R0			; divident <<= 1
   brlc   1b			; LC -= 1 if > 0 loop back to 3b
   shru   R3,R0 		; align remainder
   rtsr				; quit
