;******************************************************************************
; file:      main.asm
; content:   evaluation project for sf32b processors
;            prints "hello world"
;            runs sieve of Eratosthenes and print prime numbers up to 999
; author:    Martin Raubuch
; created:   06Jan2014
; modified:  06Jan2014
; Property of RACORS GmbH, Ginsterweg 1a, 85598 Baldham, Germany
;******************************************************************************
; ABI Conventions, allocation order
; RZ           used as stack/frame pointer
; R0-R3        data argument passing, may be destroyed by called function
; RF-RC        data/pointer arg. passing, may be destroyed by called function
; RW,RX,RY     data/pointer arg. passing, may be destroyed by called function
; R0,R1        return value
; R4-RB/RP-RV  if modified must be saved/restored by called function	
;******************************************************************************

;******************************* Program Start ********************************
   .org   PrgStart		; program start address
   jpsr   PrintHelloWorld	; PrintHelloWorld()
   jpsr   Sieve         	; Sieve(), prime number search up to 255
   jpsr   PrintPrimes         	; PrintPrimes(), print prime numbers
;******************************** Program End *********************************
   .data			; data section
EndStr:
   .asciz  "Program End"	; 0-terminated string
   .text			; text section
   jpsr   simcycle		; simcycle(), print simulation cycle number
   move   EndStr&0xffff,RF	; RF = &EndStr
   addh   EndStr>>16,RF 	; RF = &EndStr
   jpsr   puts			; puts(RF)
   move   '\n',R0		; R0 = '\n', newline
   jpsr   putchar		; putchar(R0)
ProgEnd:
   jump   ProgEnd		; endless loop at program end
;***************************** Print Hello World ******************************
; writes string "Hello World" + CR (carriage return) to stdout
; note that S (register SA) needs to be saved on the stack because a nested
; subroutine is called
;******************************************************************************
   .data			; data section
HloWldStr:
   .asciz  "Hello World"	; 0-terminated string
   .text			; text section
PrintHelloWorld:
   stlg   "S",-(RZ)		; save registers to stack
   jpsr   simcycle		; simcycle(), print simulation cycle number
   move   HloWldStr&0xffff,RF	; RF = &HloWldStr
   addh   HloWldStr>>16,RF	; RF = &HloWldStr
   jpsr   puts			; puts(RF)
   move   '\n',R0		; R0 = '\n', newline
   jpsr   putchar		; putchar(R0)
   ldlg   (RZ)+,"S"		; restore registers from stack
   rtsr				; quit
;************************* Sieve (prime number search) ************************
; searches prime numbers up to 999 using the sieve of Eratosthenes algorithm
;******************************************************************************
   .data			; data section
SieveTxt:
   .asciz  "Searchig Prime Numbers ..."	; 0-terminated string
   .text			; text section
Sieve: 
   stlg   "S",-(RZ)		; save registers to stack
   jpsr   simcycle		; simcycle(), print simulation cycle number
   move   SieveTxt&0xffff,RF	; RF = &SieveTxt
   addh   SieveTxt>>16,RF	; RF = &SieveTxt
   jpsr   puts			; puts(RF)
   move   '\n',R0		; R0 = '\n', newline
   jpsr   putchar		; putchar(R0)
   move   SieveBuf&0xffff,RF    ; buffer start address
   addh   SieveBuf>>16,RF       ; buffer start address
   move   0x0101,R0             ; R0 = 0x0101
   addh   0x0101,R0             ; R0 = 0x01010101
   move   63,LC                 ; loop count = 63, init 1008 bytes
1: stlg   R0,(RF)+		; clear 4 bytes
   stlg   R0,(RF)+		; clear 4 bytes
   stlg   R0,(RF)+		; clear 4 bytes
   stlg   R0,(RF)+		; clear 4 bytes
   brlc   1b			; LC--, if != 0 loop back to 1b
   move   SieveBuf&0xffff,RF    ; buffer start address
   addh   SieveBuf>>16,RF       ; buffer start address
   move   0,R0		        ; cleared entry value
   move   2,R1                  ; outer loop counter
1: move   2,R2                  ; inner loop counter
2: mult   R1,R2,R3		; R3 = R1*R2
   addt   1,R2			; R2 += 1
   mult   R1,R2,RC		; RC = R1*R2
   addt   1,R2			; R2 += 1
   stbt   R0,(R3,RF)	        ; clear non prime entry
   comp   1000,RC		; product > 1000 ?
   stbt   R0,(RC,RF)	        ; clear non prime entry
   brcr   2b,T			; if not loop back to 2b
   addt   1,R1			; R1 += 1
   comp   32,R1			; R1 == 32, ~SQRT(1000)
   brnz   1b			; if not loop back to 1b
   ldlg   (RZ)+,"S"		; restore registers from stack
   rtsr				; quit
;***************************** Print Prime Numbers ****************************
; prints the numbers in 'SieveBuf' as decimal numbers
;******************************************************************************
   .data			; data section
PrintTxt:
   .asciz  "Numbers found:"	; 0-terminated string
   .text			; text section
PrintPrimes: 
   stlg   "4BS",-(RZ)		; save registers to stack
   jpsr   simcycle		; simcycle(), print simulation cycle number
   move   PrintTxt&0xffff,RF	; RF = &PrintTxt
   addh   PrintTxt>>16,RF	; RF = &PrintTxt
   jpsr   puts			; puts(RF)
   move   '\n',R0		; R0 = '\n', newline
   jpsr   putchar		; putchar(R0)
   move   0,R0			; R0 = 0
   stbt   R0,NumStrBuf+3	; NumStrBuf[3] = 0, to avoid compare to undef.
   move   SieveBuf&0xffff,RB    ; RB = &SieveBuf
   addh   SieveBuf>>16,RB       ; RB = &SieveBuf
   move   2,R4			; R4 = 2, start with 2
1: ldbz   (R4,RB),R2		; R2 = next entry from SieveBuf
   move   3,R1			; R1 = 3, #of digits
   move   R4,R0			; R0 = index -> prime number
   comp   1,R2			; R2==1 ?, prime number
   brzr   3f			; if yes continue at 3f
2: addt   1,R4			; R4++
   comp   1000,R4  		; R4==1000 ? end of buffer
   brnz   1b			; if not loop back to 1b
   ldlg   (RZ)+,"4BS"		; restore registers from stack
   rtsr
3: move   NumStrBuf,RF		; RF = &NumStrBuf, number string buffer
   jpsr   DecNumToTxt		; DecNumToTxt(R0,R1,RF)
   jpsr   simcycle		; simcycle(), print simulation cycle number
   move   NumStrBuf,RF		; RF = &NumStrBuf, number string buffer
   jpsr   puts			; puts(RF)
   move   '\n',R0		; R0 = '\n', newline
   jpsr   putchar		; putchar(R0)
   jump   2b			; loop back to 2b
;********************* Decimal Number to Text conversion **********************
; converts a 32-bit integer to an ascii string, max 10 character
; leading zeroes are filled with blank ' ' characters
; input: R0 number
; input: R1 #of digits (max 10)
; input: RF pointer to text string buffer
;******************************************************************************
DecNumToTxt:
   stlg   "45ABS",-(RZ)    	; save registers
   move   R1,R4			; save #of digits
   move   R1,R5 		; save #of digits
   move   RF,RA			; save string pointer
   move   RZ,RB			; RB = digit string buffer (on stack)
1: move   10,R1			; R1 = divisor = 10
   jpsr   IntDiv3232		; IntDiv3232(R0,R1)
   addt   0x30,R0		; R0 = remainder digit ASCII code
   subf   1,RB			; RB--
   stbt   R0,(RB)		; store in string buffer
   move   R1,R0			; R0 = next divident = this quotient
   subf   1,R4			; R4--
   brnz   1b			; if != 0 loop back to 1b
   move   0x20,R1		; R1 = 0x20 = ' '
1: ldbz   (RB,1)*,R0		; R0 = ASCII character
   comp   0x30,R0		; character == '0' (leading)
   brnz   2f			; if not continue at 2f
   stbt   R1,(RA,1)*  		; store a ' ' character
   subf   1,R5  		; all digits written ?
   comp   1,R5			; R5 == 1 ?
   brnz   1b			; if not loop back
1: ldbz   (RB,1)*,R0		; R0 = ASCII character
2: stbt   R0,(RA,1)*		; write to string buffer
   subf   1,R5  		; all digits written ?
   brnz   1b			; if not loop back
   ldlg   (RZ)+,"45ABS"   	; restore registers
   rtsr
