;******************************************************************************
; file:      start.asm
; content:   evaluation project for sf32b processors
;            start code (reset entry) executed in system mode
; author:    Martin Raubuch
; created:   05Jan2014
; modified:  05Jan2014
; Property of RACORS GmbH, Ginsterweg 1a, 85598 Baldham, Germany
;******************************************************************************
; ABI Conventions, allocation order
; RZ           used as stack/frame pointer
; R0-R3        data argument passing, may be destroyed by called function
; RF-RC        data/pointer arg. passing, may be destroyed by called function
; RW,RX,RY     data/pointer arg. passing, may be destroyed by called function
; R0,R1        return value
; R4-RB/RP-RV  if modified must be saved/restored by called function	
;******************************************************************************

;******************************** Reset Entry *********************************
; typical reset entry code:
; - register IA to the program start address
; - sets the interrupt vector table start address (not used in this project)
; - initializes the stack pointer SP
; - executes an rtir to leave the system mode and jump to 'ProgStart'
;******************************************************************************
   .text                        ; switch to text segment
   .org   0x00000000            ; reset start address
   move   IrVecTab&0xffff,R0	; R0 = IrVecTab
   addh   IrVecTab>>16,R0	; R0 = IrVecTab
   move   R0,CS         	; CS.IVPT = IrVecTab
   move   PrgStart&0xffff,R0   	; R0 = PrgStart
   addh   PrgStart>>16,R0   	; R0 = PrgStart
   move   R0,IA         	; IA = PrgStart
   move   TopOfStk&0xffff,RZ   	; RZ = &TopOfStk
   addh   TopOfStk>>16,RZ   	; RZ = &TopOfStk
   rtir				; return from interrupt, switch to appl. mode
