;******************************************************************************
; file:      stdout.asm
; content:   sf32b standard out routines
; author:    Martin Raubuch
; created:   06Jan2014
; modified:  06Jan2014
; Property of RACORS GmbH, Ginsterweg 1a, 85598 Baldham, Germany
;******************************************************************************
; ABI Conventions, allocation order
; RZ           used as stack/frame pointer
; R0-R3        data argument passing, may be destroyed by called function
; RF-RC        data/pointer arg. passing, may be destroyed by called function
; RW,RX,RY     data/pointer arg. passing, may be destroyed by called function
; R0,R1        return value
; R4-RB/RP-RV  if modified must be saved/restored by called function	
;******************************************************************************

;**************************** puts (put string) *******************************
; writes the specified 0-terminated string to the standard output (sim. window)
; arg RF = string pointer
; returns nothing
;******************************************************************************
puts:
   ldbz   (RF)+,"012"		; R0/R1/R2 = next 3 bytes of string
   comp   0,R0			; R0 == 0 ? termination byte
   brzr   1f			; if yes continue at 1f
   stbt   R0,StdOut		; if not write to std-out
   comp   0,R1			; R1 == 0 ? termination byte
   brzr   1f			; if yes continue at 1f
   stbt   R1,StdOut		; if not write to std-out
   comp   0,R2			; R2 == 0 ? termination byte
   brzr   1f			; if yes continue at 1f
   stbt   R2,StdOut		; if not write to std-out
   jump   puts			; loop back to puts
1: rtsr				; quit
;************************* putchar (put character) ****************************
; writes the specified character to the standard output (simulation window)
; arg R0 = character
; returns nothing
;******************************************************************************
putchar:
   stbt   R0,StdOut		; write to std-out
   rtsr				; quit
;********************** print simulation cycle number *************************
; prints the current simulation cycle number as "@nnnnnnnnn  "
; arg none
; returns nothing
;******************************************************************************
simcycle:
   stbt   R0,SimCyc		; write to &SimCyc, trigger, data don't care
   rtsr				; quit
