/******************************************************************************
file:      sf32bu.v
content:   RACORS sf32bu 32-bit microprocessor, synthesizable Verilog RTL
           member of the sf32 family of 32-bit microprocessors
           b (base) ISA (Instruction Set Architecture)
           u (ultralight) IMA (Implementation Architecture)
created:   17Jan2014
*******************************************************************************
  Property of RACORS GmbH, Ginsterweg 1a, 85598 Baldham, Germany
  Licensing terms are specified in a separate document
******************************************************************************/
module sf32bu(CLK,RST,II,IRDY,IRN,IRQ,DBGI,STRQ,INJI,DI,DRDY,IA,IFT,IFNS,
              IFSM,IACK,DBGO,STPD,DA,DO,DBS,DWE,DSM);
  input         CLK;
  input         RST;
  input  [31:0] II;
  input         IRDY;
  input  [ 3:0] IRN;
  input         IRQ;
  input  [31:0] DBGI;
  input         STRQ;
  input         INJI;
  input  [31:0] DI;
  input         DRDY;
  output [29:0] IA;
  output        IFT;
  output        IFNS;
  output        IFSM;
  output        IACK;
  output [31:0] DBGO;
  output        STPD;
  output [31:0] DA;
  output [31:0] DO;
  output [ 3:0] DBS;
  output        DWE;
  output        DSM;
  wire   [29:0] CTU_IA;
  wire          CTU_IFT;
  wire          CTU_IFNS;
  wire          CTU_IFSM;
  wire          CTU_IACK;
  wire   [31:0] CTU_DBGO;
  wire          CTU_STPD;
  wire          CTU_IRST;
  wire   [ 3:0] CTU_CC;
  wire   [ 2:0] CTU_CSF;
  wire   [ 3:0] CTU_LIRN;
  wire          CTU_IVFT;
  wire   [ 4:0] CTU_RRS;
  wire   [31:0] CTU_DBGIR;
  wire   [29:0] CTU_PC;
  wire   [ 1:0] CTU_IRQST;
  wire   [31:0] CTU_IW;
  wire   [31:0] CTU_DIW;
  wire          CTU_DIWV;
  wire          CTU_DIWS;
  wire   [ 2:0] CTU_DAS0CT;
  wire   [ 2:0] CTU_DAS1CT;
  wire   [ 3:0] CTU_DACOCT;
  wire   [ 4:0] CTU_DALUCT;
  wire   [ 3:0] CTU_DLSCT;
  wire          CTU_DLSASL;
  wire   [ 4:0] CTU_DRRS;
  wire          CTU_AEXRD;
  wire          CTU_ALUWP;
  wire          CTU_EXCDT;
  wire   [ 1:0] CTU_RFE0;
  wire   [ 1:0] CTU_RFE1;
  wire   [ 4:0] CTU_RWS;
  wire   [ 4:0] CTU_RBWA;
  wire          CTU_RWE;
  wire   [ 1:0] CTU_DCCOPC;
  wire   [ 1:0] CTU_CCOP;
  wire          CTU_FLIV;
  wire          CTU_FEXR;
  wire   [ 2:0] CTU_IPH;
  wire   [ 2:0] CTU_NIPH;
  wire   [ 1:0] CTU_IBFL;
  wire          CTU_STALL;
  wire   [31:0] DPU_DO;
  wire   [ 3:0] DPU_CFO;
  wire   [31:0] DPU_OR0;
  wire   [31:0] DPU_MDA;
  wire   [31:0] DPU_MDO;
  wire   [ 3:0] DPU_MDBS;
  wire          DPU_MDWE;
  wire          DPU_MDSM;
  wire          DPU_MBFL;
  wire   [ 2:0] DPU_MBSWE;
  wire   [ 4:0] DPU_MBDR;
  wire   [ 4:0] DPU_MADR;
  wire          DPU_MAWE;
  wire   [ 2:0] DPU_MASWE;
  wire   [ 4:0] DPU_FMDR;
  wire          DPU_FMWE;
  wire   [ 2:0] DPU_FMSWE;
  wire          DPU_FMS1;
  wire   [ 1:0] DPU_MLST;
  wire   [31:0] RF_RP;
  wire   [31:0] RFB_DO;
  assign IA[29:0]   = CTU_IA[29:0];
  assign IFT        = CTU_IFT;
  assign IFNS       = CTU_IFNS;
  assign IFSM       = CTU_IFSM;
  assign IACK       = CTU_IACK;
  assign DBGO[31:0] = CTU_DBGO[31:0];
  assign STPD       = CTU_STPD;
  assign DA[31:0]   = DPU_MDA[31:0];
  assign DO[31:0]   = DPU_MDO[31:0];
  assign DBS[ 3:0]  = DPU_MDBS[3:0];
  assign DWE        = DPU_MDWE;
  assign DSM        = DPU_MDSM;
  sf32buCtu CTU (
  .CLK     (CLK),
  .RST     (RST),
  .IRDY    (IRDY),
  .II      (II[31:0]),
  .IRN     (IRN[3:0]),
  .IRQ     (IRQ),
  .DBGI    (DBGI[31:0]),
  .STRQ    (STRQ),
  .INJI    (INJI),
  .DPUDI   (DPU_DO[31:0]),
  .RFDI    (RF_RP[31:0]),
  .CFI     (DPU_CFO[3:0]),
  .DS0DI   (DPU_OR0[31:0]),
  .MBFL    (DPU_MBFL),
  .MBDR    (DPU_MBDR[4:0]),
  .MBWE    (DPU_MDWE),
  .MBSWE   (DPU_MBSWE[2:0]),
  .MADR    (DPU_MADR[4:0]),
  .MAWE    (DPU_MAWE),
  .MASWE   (DPU_MASWE[2:0]),
  .FMDR    (DPU_FMDR[4:0]),
  .FMWE    (DPU_FMWE),
  .FMSWE   (DPU_FMSWE[2:0]),
  .FMS1    (DPU_FMS1),
  .MLST    (DPU_MLST[1:0]),
  .IA      (CTU_IA[29:0]),
  .IFT     (CTU_IFT),
  .IFNS    (CTU_IFNS),
  .IFSM    (CTU_IFSM),
  .IACK    (CTU_IACK),
  .DBGO    (CTU_DBGO[31:0]),
  .STPD    (CTU_STPD),
  .IRST    (CTU_IRST),
  .CC      (CTU_CC[3:0]),
  .CSF     (CTU_CSF[2:0]),
  .LIRN    (CTU_LIRN[3:0]),
  .IVFT    (CTU_IVFT),
  .RRS     (CTU_RRS[4:0]),
  .DBGIR   (CTU_DBGIR[31:0]),
  .PC      (CTU_PC[29:0]),
  .IRQST   (CTU_IRQST[1:0]),
  .IW      (CTU_IW[31:0]),
  .DIW     (CTU_DIW[31:0]),
  .DIWV    (CTU_DIWV),
  .DIWS    (CTU_DIWS),
  .DAS0CT  (CTU_DAS0CT[2:0]),
  .DAS1CT  (CTU_DAS1CT[2:0]),
  .DACOCT  (CTU_DACOCT[3:0]),
  .DALUCT  (CTU_DALUCT[4:0]),
  .DLSCT   (CTU_DLSCT[3:0]),
  .DLSASL  (CTU_DLSASL),
  .DRRS    (CTU_DRRS[4:0]),
  .AEXRD   (CTU_AEXRD),
  .ALUWP   (CTU_ALUWP),
  .EXCDT   (CTU_EXCDT),
  .RFE0    (CTU_RFE0[1:0]),
  .RFE1    (CTU_RFE1[1:0]),
  .RWS     (CTU_RWS[4:0]),
  .RBWA    (CTU_RBWA[4:0]),
  .RWE     (CTU_RWE),
  .DCCOPC  (CTU_DCCOPC[1:0]),
  .CCOP    (CTU_CCOP[1:0]),
  .FLIV    (CTU_FLIV),
  .FEXR    (CTU_FEXR),
  .IPH     (CTU_IPH[2:0]),
  .NIPH    (CTU_NIPH[2:0]),
  .IBFL    (CTU_IBFL[1:0]),
  .STALL   (CTU_STALL)
  );
  sf32buDpu DPU (
  .CLK     (CLK),
  .RST     (CTU_IRST),
  .MDI     (DI[31:0]),
  .MDRDY   (DRDY),
  .RSD     (RF_RP[31:0]),
  .RFE0    (CTU_RFE0[1:0]),
  .RFE1    (CTU_RFE1[1:0]),
  .IW      (CTU_DIW[31:0]),
  .IWSM    (CTU_DIWS),
  .DBGI    (CTU_DBGIR[31:0]),
  .CC      (CTU_CC[3:0]),
  .EXRD    (CTU_AEXRD),
  .ALUWP   (CTU_ALUWP),
  .S0CT    (CTU_DAS0CT[2:0]),
  .S1CT    (CTU_DAS1CT[2:0]),
  .COCT    (CTU_DACOCT[3:0]),
  .DCCOPC  (CTU_DCCOPC[1:0]),
  .CCOP    (CTU_CCOP[1:0]),
  .DRRS    (CTU_DRRS[4:0]),
  .RBWA    (CTU_RBWA[4:0]),
  .RWE     (CTU_RWE),
  .LSCT    (CTU_DLSCT[3:0]),
  .LSASL   (CTU_DLSASL),
  .IVFT    (CTU_IVFT),
  .LIRN    (CTU_LIRN[3:0]),
  .PC      (CTU_PC[29:0]),
  .IRQST   (CTU_IRQST[1:0]),
  .EXCDT   (CTU_EXCDT),
  .OPCT    (CTU_DALUCT[4:0]),
  .DO      (DPU_DO[31:0]),
  .CFO     (DPU_CFO[3:0]),
  .OR0     (DPU_OR0[31:0]),
  .MDA     (DPU_MDA[31:0]),
  .MDO     (DPU_MDO[31:0]),
  .MDBS    (DPU_MDBS[3:0]),
  .MDWE    (DPU_MDWE),
  .MDSM    (DPU_MDSM),
  .MBFL    (DPU_MBFL),
  .MBSWE   (DPU_MBSWE[2:0]),
  .MBDR    (DPU_MBDR[4:0]),
  .MADR    (DPU_MADR[4:0]),
  .MAWE    (DPU_MAWE),
  .MASWE   (DPU_MASWE[2:0]),
  .FMDR    (DPU_FMDR[4:0]),
  .FMWE    (DPU_FMWE),
  .FMSWE   (DPU_FMSWE[2:0]),
  .FMS1    (DPU_FMS1),
  .MLST    (DPU_MLST[1:0])
  );
  sf32buRf RF (
  .CLK   (CLK),
  .RST   (CTU_IRST),
  .DRRS  (CTU_DRRS[4:0]),
  .CC    (CTU_CC[3:0]),
  .CSF   (CTU_CSF[2:0]),
  .RBDO  (RFB_DO[31:0]),
  .RBDI  (DPU_DO[31:0]),
  .RBWA  (CTU_RBWA[4:0]),
  .RBWE  (CTU_RWE),
  .RP    (RF_RP[31:0])
  );
  sf32buRfb RFB (
  .CLK  (CLK),
  .RAD  (CTU_RRS[4:0]),
  .DI   (DPU_DO[31:0]),
  .WAD  (CTU_RBWA[4:0]),
  .WE   (CTU_RWE),
  .DO   (RFB_DO[31:0])
  );
endmodule
module sf32buCtu(CLK,RST,II,IRDY,IRN,IRQ,DBGI,STRQ,INJI,DPUDI,CFI,DS0DI,
                 RFDI,MBFL,MBDR,MBWE,MBSWE,MADR,MAWE,MASWE,FMDR,FMWE,FMSWE,
                 FMS1,MLST,IA,IFT,IFNS,IFSM,IACK,DBGO,STPD,IRST,CC,CSF,
                 LIRN,IVFT,RRS,DBGIR,PC,IRQST,IW,DIW,DIWV,DIWS,DAS0CT,DAS1CT,
                 DACOCT,DALUCT,DLSCT,DLSASL,DRRS,AEXRD,ALUWP,EXCDT,RFE0,RFE1,
                 RWS,RBWA,RWE,DCCOPC,CCOP,FLIV,FEXR,IPH,NIPH,IBFL,STALL);
  input         CLK;
  input         RST;
  input  [31:0] II;
  input         IRDY;
  input  [ 3:0] IRN;
  input         IRQ;
  input  [31:0] DBGI;
  input         STRQ;
  input         INJI;
  input  [31:0] DPUDI;
  input  [ 3:0] CFI;
  input  [31:0] DS0DI;
  input  [31:0] RFDI;
  input         MBFL;
  input  [ 4:0] MBDR;
  input         MBWE;
  input  [ 2:0] MBSWE;
  input  [ 4:0] MADR;
  input         MAWE;
  input  [ 2:0] MASWE;
  input  [ 4:0] FMDR;
  input         FMWE;
  input  [ 2:0] FMSWE;
  input         FMS1;
  input  [ 1:0] MLST;
  output [29:0] IA;
  output        IFT;
  output        IFNS;
  output        IFSM;
  output        IACK;
  output [31:0] DBGO;
  output        STPD;
  output        IRST;
  output [ 3:0] CC;
  output [ 2:0] CSF;
  output [ 3:0] LIRN;
  output        IVFT;
  output [ 4:0] RRS;
  output [31:0] DBGIR;
  output [29:0] PC;
  output [ 1:0] IRQST;
  output [31:0] IW;
  output [31:0] DIW;
  output        DIWV;
  output        DIWS;
  output [ 2:0] DAS0CT;
  output [ 2:0] DAS1CT;
  output [ 3:0] DACOCT;
  output [ 4:0] DALUCT;
  output [ 3:0] DLSCT;
  output        DLSASL;
  output [ 4:0] DRRS;
  output        AEXRD;
  output        ALUWP;
  output        EXCDT;
  output [ 1:0] RFE0;
  output [ 1:0] RFE1;
  output [ 4:0] RWS;
  output [ 4:0] RBWA;
  output        RWE;
  output [ 1:0] DCCOPC;
  output [ 1:0] CCOP;
  output        FLIV;
  output        FEXR;
  output [ 2:0] IPH;
  output [ 2:0] NIPH;
  output [ 1:0] IBFL;
  output        STALL;
  reg    [29:0] IA;
  reg           IFT;
  reg           IFNS;
  reg           IFSM;
  reg           IACK;
  reg    [31:0] DBGO;
  reg           STPD;
  reg           IRST;
  reg    [ 3:0] CC;
  reg    [ 2:0] CSF;
  reg    [ 3:0] LIRN;
  reg           IVFT;
  reg    [ 4:0] RRS;
  reg    [31:0] DBGIR;
  reg    [29:0] PC;
  reg    [ 1:0] IRQST;
  reg    [31:0] IW;
  reg    [31:0] DIW;
  reg           DIWV;
  reg           DIWS;
  reg    [ 2:0] DAS0CT;
  reg    [ 2:0] DAS1CT;
  reg    [ 3:0] DACOCT;
  reg    [ 4:0] DALUCT;
  reg    [ 3:0] DLSCT;
  reg           DLSASL;
  reg    [ 4:0] DRRS;
  reg           AEXRD;
  reg           ALUWP;
  reg           EXCDT;
  reg    [ 1:0] RFE0;
  reg    [ 1:0] RFE1;
  reg    [ 4:0] RWS;
  reg    [ 4:0] RBWA;
  reg           RWE;
  reg    [ 1:0] DCCOPC;
  reg    [ 1:0] CCOP;
  reg           FLIV;
  reg           FEXR;
  reg    [ 2:0] IPH;
  reg    [ 2:0] NIPH;
  reg    [ 1:0] IBFL;
  reg           STALL;
  reg    [ 1:0] RSYN;
  reg    [ 3:0] CCS;
  reg           IWSM;
  reg    [31:0] IB0;
  reg    [31:0] IB1;
  reg    [ 1:0] IBSM;
  reg    [29:0] MAIA;
  reg    [ 1:0] MAST;
  reg           LCTST;
  reg           NSIF;
  reg           DSOIR;
  reg           EXRQ;
  reg           LSTRQ;
  reg           LINJI;
  reg           LDBRQ;
  reg           INDB;
  reg           RWES;
  reg    [ 3:0] SRWES;
  reg    [ 1:0] CCOPS;
  reg    [ 3:0] EXCDS;
  reg           LCTDS;
  reg           DLDSTM;
  reg    [ 2:0] DSRLD;
  reg           DRRE;
  reg    [ 2:0] DRWE;
  reg    [ 3:0] DSRWE;
  reg           DCDEX;
  reg           DDBGC;
  reg           DBGWE;
  reg    [ 4:0] SHCT;
  reg    [29:0] BTIA;
  reg    [29:0] BOFS;
  reg    [ 2:0] IACT;
  reg    [ 2:0] FLCT;
  reg    [ 1:0] CCSCT;
  reg           STPCT;
  reg           BCDT;
  reg    [ 1:0] PCCT;
  reg           IIV;
  reg           IWCS;
  reg    [ 2:0] QDIW;
  reg           CCPO;
  reg           LCPO;
  reg           TAPO;
  reg           SAPO;
  reg           SOIR;
  reg           RFST;
  reg    [ 2:0] NCCSF;
  reg    [ 1:0] NCMAST;
  reg           NCNSIF;
  reg    [ 1:0] NCIRQST;
  reg    [ 1:0] NCIBFL;
  reg           NCLDBRQ;
  reg           SQFFI;
  reg    [ 1:0] LCTLD;
  reg           RFNXI;
  reg           STRR;
  reg           STRW;
  reg    [ 4:0] IMRS;
  reg    [ 4:0] DMRS;
  reg    [31:0] RFCM;
  reg    [ 4:0] RRSC;
  reg    [ 2:0] RWEC;
  reg           CDEX;
  reg           SCVL;
  reg           DBGC;
  reg           LSSL;
  reg    [ 2:0] AS0CT;
  reg    [ 2:0] AS1CT;
  reg    [ 3:0] ACOCT;
  reg    [ 4:0] ALUCT;
  reg    [ 1:0] CCOPC;
  reg    [ 3:0] LSCT;
  reg           LSASL;
  reg           LDSTM;
  reg           LCTD;
  reg    [ 4:0] NCRWS;
  reg           NCRWES;
  reg    [ 4:0] NCSHCT;
  reg               c;     
  reg               o;     
  reg               z;     
  reg               n;     
  task idec_jump_ia29; begin
    IACT[0] = 1;
    NCNSIF  = 1;
    if(IW[4])                     
      if(MAWE & (DRWE!=0)) begin
        FEXR   = 0;
        NCNSIF = 0;
      end
      else begin
        RWEC     = 5;             
        AS0CT[1] = 1;             
        AS1CT[2] = 1;             
      end
  end endtask
  task idec_jump_ta; begin
    if(IPH == 0) begin
      if(TAPO == 0) begin
        NIPH = 1;                 
        AS0CT[0] = 1;             
        RRSC     = 5'h14;         
      end
      FEXR = 0;
    end
    if(IPH == 1) begin
      NIPH    = 0;                
      IACT[2] = 1;
      NCNSIF  = 1;
      if(IW[4]) begin             
        RWEC     = 5;             
        AS0CT[1] = 1;             
        AS1CT[2] = 1;             
      end
    end
  end endtask
  task idec_blc; begin
    FEXR = ~LCPO;
    if(FEXR) begin
      if(LCTST) begin
        IACT[1] = 1;
        NCNSIF  = 1;
      end
      else
        SQFFI  = 1;
      RWEC[2]  = 1;               
      RRSC[4]  = 1;               
      AS0CT[0] = 1;               
      ACOCT[0] = 1;               
      AS1CT[1] = 1;               
      ALUCT[0] = 1;               
    end
  end endtask
  task idec_return; begin
    if(IPH == 0) begin
      FEXR = 0;                   
      if(IW[4] | ~SAPO) begin
        NIPH     = 1;             
        AS0CT[0] = 1;             
        RRSC = {3'h7,IW[4]?2'd2:2'd1}; 
      end
    end
    if(IPH == 1) begin
      NIPH    = 0;                
      IACT[2] = 1;                
      NCNSIF  = 1;                
      if(IW[4])                   
        CCSCT[1] = 1;             
    end
  end endtask
  task idec_cdbranch; begin
    FEXR = ~IRQST[1] & ~CCPO;
    if(FEXR & BCDT) begin
      IACT[1] = 1;
      NCNSIF  = 1;
    end
    else
      SQFFI = 1;
  end endtask
  task idec_stop; begin
    STPCT = 1;
    FEXR  = IRQST == 0;
  end endtask
  task idec_sbadmv; begin
    RWEC[0]  = 1;                 
    CCOPC[0] = ~IW[5];            
    CCOPC[1] = IW[5:4] == 1;      
    CDEX     = 1;                 
    AS0CT[1] = 1;                 
    if(IW[5]) begin               
      ACOCT = 15            ;     
      ALUCT[2] = 1;               
    end
    else begin                    
      RRSC[1]  = 1;               
      AS1CT[0] = 1;               
      ACOCT = 6;                  
      ALUCT[0] = IW[3];           
    end
  end endtask
  task idec_compcon; begin
    CCOPC    = {IW[4],1'b1};      
    RRSC[1]  = 1;                 
    AS1CT[0] = 1;                 
    ACOCT    = 13;                
    AS0CT[1] = 1;                 
  end endtask
  task idec_compreg; begin
    if(IPH == 1) begin
      NIPH     = 0;               
      RRSC[1]  = 1;               
      AS1CT[0] = 1;               
      CCOPC    = {IW[4],1'b1};    
    end
    else begin
      NIPH     = 1;               
      RRSC[0]  = 1;               
      AS0CT[0] = 1;               
    end
  end endtask
  task idec_dual; begin
    RWEC[0]  = 1;                 
    CDEX     = 1;                 
    RRSC[0]  = 1;                 
    AS0CT[0] = 1;                 
    AS1CT[1] = 1;                 
    ACOCT[0] = IW[5];             
    case(IW[5:3])
      0: ALUCT[2] = 1;            
      3: ALUCT[1] = 1;            
      4: ALUCT = 5'h13;           
      5: ALUCT[2:1] = 3;          
      6: ALUCT[1:0] = 3;          
      7: ALUCT[2:0] = 7;          
      default: ;
    endcase
  end endtask
  task idec_shfbit; begin
    CDEX   = 1;                   
    ALUCT[4:2] = IW[5] ? 3'd5:3'd6; 
    ALUCT[1:0] = IW[4:3];         
    if(IPH == 0)
      if((IW>>25) & 1) begin      
        NIPH    = 1;              
        RRSC[0] = 1;              
      end
      else begin
        RRSC[1]  = 1;             
        AS1CT[0] = 1;             
        if(~IW[5] & (IW[20:17]!=0)) 
          NIPH = 2;               
        else begin
          RWEC[0]  = IW[5:3] != 5; 
          CCOPC[0] = IW[5:3] == 5; 
        end
      end
    if(IPH == 1) begin
      if(IW[5]) begin
        NIPH     = 0;             
        RRSC[1]  = 1;             
        AS1CT[0] = 1;             
        RWEC[0]  = IW[4:3] != 1;  
        CCOPC[0] = IW[4:3] == 1;  
      end
      else
        NIPH = 3;                 
    end
    if((IPH==2) & (SHCT[4:1]==0)) begin
      NIPH    = 0;                
      RWEC[0] = 1;                
    end
    if(IPH == 3) begin
      RRSC[1]  = 1;               
      AS0CT[0] = 1;               
      AS1CT[0] = 1;               
      if(SHCT[4:1] == 0) begin
        NIPH    = 0;              
        RWEC[0] = 1;              
        if(~SHCT[0])              
          ALUCT = 4;              
      end
      else
        NIPH = 2;                 
    end
  end endtask
  task idec_triacon; begin
    RWEC[0] = 1;                  
    if((IW[5:4]==0) | (IW[5:3]==5))
      CCOPC[0] = 1;               
    AS0CT[1] = 1;                 
    ACOCT[3:2] = IW[5:3]==3 ? 2'd1:2'd2; 
    RRSC[1]  = 1;                 
    AS1CT[0] = 1;                 
    case(IW[5:3])
      1,
      3: ALUCT[0] = 1;            
      5: ALUCT = 5'h11;           
      6: ALUCT = 5'h12;           
      7: ALUCT = 5'h13;           
      default: ;
    endcase
  end endtask
  task idec_triareg; begin
    CDEX = 1;                     
    case(IW[5:3])
      1,
      3: ALUCT[0] = 1;            
      5: ALUCT = 5'h11;           
      6: ALUCT = 5'h12;           
      7: ALUCT = 5'h13;           
      default: ;
    endcase
    if(IPH == 1) begin
      NIPH    = 0;                
      RWEC[0] = 1;                
      if(~IW[5] | (IW[5:3]==5))
        CCOPC[0] = 1;             
      CCOPC[1] = IW[5:4]==1;      
      RRSC[1]  = 1;               
      AS1CT[0] = 1;               
    end
    else begin
      NIPH     = 1;               
      RRSC[0]  = 1;               
      AS0CT[0] = 1;               
    end
  end endtask
  task idec_ldst; begin
    if(IW[27] & IW[15] & IW[0])   
      CDEX  = 0;                  
    else
      CDEX  = 1;                  
    LSCT[2:0] = IW[5:3];          
    ALUCT[0] = 1;                 
    if(IPH == 0) begin
      NIPH = 1;                   
      case({IW[15],IW[0]})
        0: begin 
             ACOCT      = 4'hb;   
             AS0CT[1]   = 1;      
             ALUCT[2:0] = 4;      
           end
        1,
        2: begin
             AS0CT[1] = 1;        
             ACOCT    = 7;        
             RRSC[2]  = 1;        
             AS1CT[0] = 1;        
           end
        3: begin
             RRSC[2]  = 1;        
             AS1CT[0] = 1;        
             if(IW[27]) begin     
               AS0CT[1] = 1;      
               ACOCT[1:0] = {1'b1,IW[26]}; 
               LDSTM = 1;         
         if((IW[30:28]==0)&(IW[25:16]==0)&(IW[10:6]==0))
           LDSTM = 0;
         if(~IW[31]&(IW[29:28]==0)&(IW[25:16]==0)&(IW[10:6]==0))
           LDSTM = 0;
         if((IW[31:30]==0)&~IW[28]&(IW[25:16]==0)&(IW[10:6]==0))
           LDSTM = 0;
         if((IW[31:29]==0)&(IW[25:16]==0)&(IW[10:6]==0))
           LDSTM = 0;
         if((IW[31:28]==0)&(IW[24:16]==0)&(IW[10:6]==0))
           LDSTM = 0;
         if((IW[31:28]==0)&~IW[25]&(IW[23:16]==0)&(IW[10:6]==0))
           LDSTM = 0;
         if((IW[31:28]==0)&(IW[25:24]==0)&(IW[22:16]==0)&(IW[10:6]==0))
           LDSTM = 0;
         if((IW[31:28]==0)&(IW[25:23]==0)&(IW[21:16]==0)&(IW[10:6]==0))
           LDSTM = 0;
         if((IW[31:28]==0)&(IW[25:22]==0)&(IW[20:16]==0)&(IW[10:6]==0))
           LDSTM = 0;
         if((IW[31:28]==0)&(IW[25:21]==0)&(IW[19:16]==0)&(IW[10:6]==0))
           LDSTM = 0;
         if((IW[31:28]==0)&(IW[25:20]==0)&(IW[18:16]==0)&(IW[10:6]==0))
           LDSTM = 0;
         if((IW[31:28]==0)&(IW[25:19]==0)&(IW[17:16]==0)&(IW[10:6]==0))
           LDSTM = 0;
         if((IW[31:28]==0)&(IW[25:18]==0)&~IW[16]&(IW[10:6]==0))
           LDSTM = 0;
         if((IW[31:28]==0)&(IW[25:17]==0)&(IW[10:6]==0))
           LDSTM = 0;
         if((IW[31:28]==0)&(IW[25:16]==0)&(IW[9:6]==0))
           LDSTM = 0;
         if((IW[31:28]==0)&(IW[25:16]==0)&~IW[10]&(IW[8:6]==0))
           LDSTM = 0;
         if((IW[31:28]==0)&(IW[25:16]==0)&(IW[10:9]==0)&(IW[7:6]==0))
           LDSTM = 0;
         if((IW[31:28]==0)&(IW[25:16]==0)&(IW[10:8]==0)&~IW[6])
           LDSTM = 0;
         if((IW[31:28]==0)&(IW[25:16]==0)&(IW[10:7]==0))
           LDSTM = 0;
	     end
             else                 
               NIPH = 2;          
           end
      endcase
    end
    if(IPH == 1) begin
      if(~IW[15] & IW[0])         
        RWEC[1] = 1;              
      if((IW[27:26]!=1) & IW[15] & IW[0]) 
        RWEC[1] = 1;              
      if(IW[27] & IW[15] & IW[0]) 
        RRSC[3] = 1;              
      else
        RRSC[1] = 1;              
      NIPH  = 0;                  
      LSCT[3] = 1;                
      if(~IW[0])                  
        LSASL = 1;                
      if(IW[26] & IW[15] & IW[0]) 
        LSASL = 1;                
    end
    if(IPH == 2) begin            
      RRSC[0]  = 1;               
      AS0CT[0] = 1;               
      NIPH     = 1;               
    end
  end endtask
  task idec_multcon; begin
    if(IPH == 0) begin
      NIPH     = 1;               
      ACOCT    = {3'b100,IW[3]};  
      AS0CT[1] = 1;               
      RRSC[1]  = 1;               
      AS1CT[0] = 1;               
      ALUCT = {4'h6,IW[3]};       
    end
    if(IPH == 1) begin
      NIPH    = 0;                
      RWEC[0] = 1;                
    end
  end endtask
  task idec_multreg; begin
    CDEX = 1;                     
    if(IPH == 0) begin
      NIPH      = 1;              
      RRSC[0]  = 1;               
      AS0CT[0] = 1;               
    end
    if(IPH == 1) begin
      NIPH     = 2;               
      RRSC[1]  = 1;               
      AS1CT[0] = 1;               
      ALUCT    = {2'b10,IW[4:3]}; 
    end
    if(IPH == 2)
      NIPH = 3;                   
    if(IPH == 3) begin
      if(IW[4])                   
        NIPH = 4;                 
      else begin
        NIPH    = 0;              
        RWEC[0] = 1;              
      end
    end
    if(IPH == 4) begin
      NIPH    = 0;                
      RWEC[0] = 1;                
    end
  end endtask
  task idec_debug; begin
    ALUCT[2] = 1;                 
    if(IW[3]) begin               
      AS0CT[2] = 1;               
      RWEC[0]  = 1;               
    end
    else begin                    
      RRSC[0]  = 1;               
      AS0CT[0] = 1;               
      DBGC     = 1;               
    end
  end endtask
  always @(RSYN)
    IRST = RSYN[0];
  always @(IRQST or MBFL)
    IVFT = (IRQST==2) & ~MBFL;
  always @(IBFL or MAST)
    QDIW = IBFL + MAST[0];
  always @(MAST or IRDY)
    IIV = MAST[0] & IRDY;
  always @(IW)
    LSSL = IW[5:3] > 4;
  always @(IW or CC) begin
    BCDT = 0;
    c = CC[0];
    o = CC[1];
    z = CC[2];
    n = CC[3];
    case(IW[7:5])
      0: BCDT = ~c;
      1: BCDT = ~o;
      2: BCDT = ~z;
      3: BCDT = ~n;
      4: BCDT =  c | z;
      5: BCDT = (n & ~o) | (~n &  o);
      6: BCDT =  z |  (n & ~o) | (~n &  o);
      default: ;
    endcase
    BCDT = BCDT ^ IW[4];
  end
  always @(CC or EXCDS) begin
    EXCDT = 1'b0;
    c = CC[0];
    o = CC[1];
    z = CC[2];
    n = CC[3];
    if(EXCDS[3])
      case(EXCDS[2:0])
        4'h0: EXCDT =  c | z;
        4'h1: EXCDT = ~c & ~z;
        4'h2: EXCDT = (n & ~o) | (~n & o);
        4'h3: EXCDT = (n & o) | (~n & ~o);
        4'h4: EXCDT = z | (n & ~o) | (~n & o);
        4'h5: EXCDT = ~z & ((n & o) | (~n & ~o));
        4'h7: EXCDT = 1'b1;
        default: ;
      endcase
    else
      EXCDT = CC[EXCDS[2:1]] ^ ~EXCDS[0];
  end
  always @(RWES or EXCDT or FMWE or RWS or FMDR)
    ALUWP = RWES & EXCDT & FMWE & (RWS==FMDR);
  always @(RWES or DAS0CT or EXCDT or RWS or DRRS or FMWE or FMDR) begin
    RFE0[0] = RWES & DAS0CT[0] & EXCDT & (RWS==DRRS);
    RFE0[1] = FMWE & (FMDR==DRRS);
  end
  always @(RWES or DAS1CT or EXCDT or RWS or DRRS or SHCT or RFE0) begin
    RFE1[0] = (RWES&DAS1CT[0]&EXCDT&(RWS==DRRS)) | ((SHCT!=0)&~DAS1CT[0]);
    RFE1[1] = RFE0[1];
  end
  always @(DRRE or RWES or DRRS or RWS or DIW or FMWE or FMDR or CCOPS or
	   MAWE or MADR or MBFL or MBWE or MBDR) begin
    STRR = 0;
    if(DRRE) begin
      STRR = RWES & (DRRS==RWS)  & (DIW[27:25]==5) & (DIW[2:0]==0);
      STRR = STRR | (FMWE &(DRRS==FMDR)&(DIW[27:25]==5)&(DIW[2:0]==0));
      STRR = STRR | (CCOPS[0]&(DRRS==5'h19));
      STRR = STRR | (MAWE&(DRRS==MADR));
      STRR = STRR | (MBFL&~MBWE&(DRRS==MBDR));
    end
  end
  always @(RWES or MAWE or RWS or MADR or MBFL or MBWE or MBDR) begin
    STRW = 0;
    if(RWES) begin
      STRW = MAWE & (RWS==MADR);
      STRW = STRW | (MBFL & ~MBWE & (RWS==MBDR));
    end
  end
  always @(STRR or STRW or FMWE or ALUWP or RWES or CCOPS or DLSCT or IVFT or
           MBFL or DALUCT) begin
    STALL = STRR | STRW | (FMWE&~ALUWP&(RWES|CCOPS[0]));
    STALL = STALL | (DLSCT[3]&MBFL) | ((DALUCT[4:2]==5)&MBFL);
  end
  always @(LCTDS)
    LCTD = LCTDS; 
  always @(FMWE or ALUWP or FMDR or RWS)
    RBWA = FMWE & ~ALUWP ? FMDR : RWS;
  always @(IRQST or FMWE or FMDR)
    SOIR = (IRQST==3) & FMWE & (FMDR==5'h1f);
  always @(IBFL or MAST or DRRE or RWES or CCOPS or LCTDS or STALL or MLST or
	   DIWV or DRWE or DLSCT or MBFL) begin
    RFST = (IBFL==0) & ~MAST[0] & ~DRRE & ~(RWES|CCOPS[0]|LCTDS);
    RFST = RFST & ~STALL & (MLST==0) & ~DIWV & (DRWE==0) & ~DLSCT[3] & ~MBFL;
  end
  always @(SHCT or DALUCT or IW or IPH or RFDI or IBFL) begin
    NCSHCT = SHCT;
    if((SHCT!=0) & (DALUCT[4:3]==3) & (IPH!=3))
      NCSHCT = NCSHCT - 1'b1;
    if(IW[27] & ~IW[26] & ~IW[5] & (IW[2:0]==0) & (IBFL!=0)) begin
      if((IPH==0) & ~IW[25])
        NCSHCT = IW[20:16];
      if(IPH == 1)
        NCSHCT = RFDI[4:0];
    end
  end
  always @(DSRWE or SRWES or DSRLD or MBSWE or MASWE or FMSWE)
    LCPO = DSRWE[0] | SRWES[0] | DSRLD[0] | MBSWE[0] | MASWE[0] | FMSWE[0];
  always @(DSRWE or SRWES or DSRLD or MBSWE or MASWE or FMSWE)
    TAPO = DSRWE[1] | SRWES[1] | DSRLD[1] | MBSWE[1] | MASWE[1] | FMSWE[1];
  always @(DSRWE or SRWES or DSRLD or MBSWE or MASWE or FMSWE)
    SAPO = DSRWE[2] | SRWES[2] | DSRLD[2] | MBSWE[2] | MASWE[2] | FMSWE[2];
  always @(DSRWE or SRWES or DCCOPC or CCOPS)
    CCPO = DSRWE[3] | SRWES[3] | (DCCOPC!=0) | (CCOPS!=0);
  always @(IW or IBFL or FLIV or IRQST or PCCT or NSIF or IIV or TAPO or
           IPH or SAPO or CCPO  or RFDI or SHCT or LCTST or BCDT or LSSL or
           DSOIR or SOIR or STALL or LCPO or MAWE or DRWE) begin
    FLIV   = 0;
    FEXR   = 1;
    IACT   = 0;
    FLCT   = 0;
    CCSCT  = 0;
    STPCT  = 0;
    PCCT   = 0;
    NCNSIF = 0;
    SQFFI  = 0;
    RRSC   = 0;
    RWEC   = 0;
    CCOPC  = 0;
    CDEX   = 0;
    DBGC   = 0;
    AS0CT  = 0;
    AS1CT  = 0;
    ACOCT  = 0;
    ALUCT  = 0;
    LSCT   = 0;
    LSASL  = 0;
    LDSTM  = 0;
    NIPH   = IPH;
    if(IBFL != 0)
      if(IW[2:0] == 6) begin                
        FLIV = 1;
        if(IW[3])
	  idec_jump_ia29;
	else
	  case(IW[8:7])
            0: case(IW[6:4])
                 0,
                 1: idec_jump_ta;
                 2,
                 3: idec_return;
                 4: idec_stop;
                 6: PCCT[0] = 1;
                 7: PCCT[1] = 1;
                 default: ;
               endcase
            1: begin
                 FLCT  = {1'b1,IW[5:4]};
	         SQFFI = 1;
	         FEXR  = FEXR & ~SAPO & (IRQST==0);
	       end
            default: if(IW[7:5] == 7)
                       idec_blc;
                     else
                       idec_cdbranch;
	  endcase
      end
      else begin                            
        if(IW[2])                           
          idec_ldst;        
        else                                
          case(IW[1:0])
            0: case(IW[27:25])
	         0: idec_multreg;
	         1: idec_triareg;
	         2,
	         3: idec_dual;
	         4,
	         5: idec_shfbit;
	         6: idec_debug;
	         7: idec_compreg;
	         default: ;
	       endcase
            1: idec_sbadmv;
            2: idec_triacon;
            3: if(IW[5])
                 idec_compcon;
               else
                 idec_multcon;
          endcase
      end
    NCNSIF = (NCNSIF&FLIV)| PCCT[1]  | DSOIR | (NSIF&~IIV);
    CCSCT[0] = SOIR;
    if(STALL & (RWEC[2]|RRSC[4])) begin
      FEXR   = 0;
      NCNSIF = 0;
    end
  end
  always @(FMWE or RWES or EXCDT or STRW or RWS or CSF or INDB or RWE) begin
    RWE  = FMWE | (RWES&EXCDT&~STRW);
    SCVL = 0;
    if(~CSF[0] & ~INDB & RWE & (RWS[4:3]==3) & (RWS[1:0]==2)) begin
      SCVL = 1;
      RWE  = 0;
    end
  end
  always @(EXCDT or STRW or FMWE or ALUWP or CCOPS)
    CCOP = EXCDT & ~STRW & (~FMWE|ALUWP) ? CCOPS : 2'd0;
  always @(STALL)
    AEXRD = ~STALL;
  always @(NIPH or STALL or FEXR or DLDSTM)
    RFNXI = (NIPH==0) & ~STALL & FEXR & ~DLDSTM;
  always @(IRST or QDIW or FLIV or SQFFI or FEXR or NCNSIF or NSIF or IRQST or
	   LSTRQ or LDBRQ) begin
    IFT = ~IRST & (QDIW<3) & (IRQST==0);
    if(FLIV & ~SQFFI)
      IFT = FEXR & NCNSIF & ~NSIF;
    if(LSTRQ | LDBRQ)
      IFT = 0;
  end
  always @(CSF or CCSCT)
    IFSM = CSF[0] & ~CCSCT[1];
  always @(IW or PC) begin
    BOFS[13: 0] = IW[23:10];
    BOFS[29:14] = IW[23] ? 16'hffff : 16'h0;
    BTIA = PC + BOFS;
  end
  always @(IBFL or RFNXI)
    IWCS = (IBFL!=0) & RFNXI;
  always @(IRQST or LSTRQ or LDBRQ or STPCT or EXRQ or CSF or IRQ or FLCT or
           RFST or MBFL or FMWE or FMDR) begin
    NCIRQST = IRQST;
    case(IRQST)
      0: if(~LSTRQ & ~LDBRQ & ~STPCT)
           if(EXRQ & ~CSF[0])
             NCIRQST = 1;
           else if(IRQ & (CSF[1:0]==2) & ~(FLCT[0]&FLCT[2]))
             NCIRQST = 1;
      1: begin
           if(RFST)
             NCIRQST = 2;
           if(STPCT)
             NCIRQST = 0;
	 end
      2: NCIRQST = MBFL ? 2'd1 : 2'd3;
      3: if(FMWE & (FMDR==5'h1f))
           NCIRQST = 0;
    endcase
  end
  always @(NCNSIF or NSIF)
    IFNS = NCNSIF & ~NSIF;
  always @(IACT or IW or BTIA or RFDI or MAST or MAIA) begin
    IA = 0;
    if(IACT) begin
      if(IACT[0])
        IA[26:0] = IW[31:5]; 
      if(IACT[1])
        IA = BTIA;
      if(IACT[2])
        IA = RFDI[31:2];
    end
    else
      IA = MAST[0] ? MAIA + 1'b1 : MAIA;
  end
  always @(MAST or IIV or NCNSIF or NSIF or IFT or STPCT or FEXR or SOIR or
           FLIV or IFSM) begin
    NCMAST = MAST;
    if(~MAST[0] | IIV | (NCNSIF&~NSIF) | (FLIV&~FEXR))
      NCMAST = {IFT&IFSM,IFT};
    if(STPCT & FEXR)
      NCMAST = 0;
  end
  always @(CSF or CCSCT or SOIR or FLCT or FEXR or RWE or RWS or DS0DI or
           INDB or STALL) begin
    NCCSF = CSF;
    if(CCSCT[1])
      NCCSF[0] = 0;
    if(SOIR)
      NCCSF[0] = 1;
    if(FLCT[2] & FEXR & ~STALL)
      case(FLCT[1:0])
        0: NCCSF[1] = 1;
        1: NCCSF[1] = 0;
        2: NCCSF[1] = CSF[2];
        3: begin
             NCCSF[2] = CSF[1];
             NCCSF[1] = 0;
           end
      endcase
  end
  always @(IBFL or IIV or IWCS or NCNSIF or NSIF or SOIR or STPCT or FEXR or
	   LINJI or FLCT or IRQST) begin
    NCIBFL = IBFL + IIV - IWCS;
    if((NCNSIF&~NSIF&FEXR) | SOIR | (STPCT&FEXR) |
       (FLCT[0]&FLCT[2]&(IRQST==1)))
      NCIBFL = 0;
    if(LINJI)
      NCIBFL = 1;
  end
  always @(STPCT or FEXR or LDBRQ or LSTRQ)
    NCLDBRQ = (STPCT&FEXR) | (LDBRQ&~LSTRQ);
  always @(DS0DI) begin
    LCTLD[0] = (DS0DI[31:2]==0) & (DS0DI[1:0]!=3);
    LCTLD[1] = LCTLD[0] & ~DS0DI[1];
  end
  always @(IW) begin
    RFCM = 32'hffffffff;
         if(IW[31]) begin IMRS = 5'h1d; DMRS = 5'h13; RFCM[31] = 0; end
    else if(IW[30]) begin IMRS = 5'h1c; DMRS = 5'h12; RFCM[30] = 0; end
    else if(IW[29]) begin IMRS = 5'h18; DMRS = 5'h11; RFCM[29] = 0; end
    else if(IW[28]) begin IMRS = 5'h00; DMRS = 5'h10; RFCM[28] = 0; end
    else if(IW[25]) begin IMRS = 5'h01; DMRS = 5'h0b; RFCM[25] = 0; end
    else if(IW[24]) begin IMRS = 5'h02; DMRS = 5'h0a; RFCM[24] = 0; end
    else if(IW[23]) begin IMRS = 5'h03; DMRS = 5'h09; RFCM[23] = 0; end
    else if(IW[22]) begin IMRS = 5'h04; DMRS = 5'h08; RFCM[22] = 0; end
    else if(IW[21]) begin IMRS = 5'h05; DMRS = 5'h07; RFCM[21] = 0; end
    else if(IW[20]) begin IMRS = 5'h06; DMRS = 5'h06; RFCM[20] = 0; end
    else if(IW[19]) begin IMRS = 5'h07; DMRS = 5'h05; RFCM[19] = 0; end
    else if(IW[18]) begin IMRS = 5'h08; DMRS = 5'h04; RFCM[18] = 0; end
    else if(IW[17]) begin IMRS = 5'h09; DMRS = 5'h03; RFCM[17] = 0; end
    else if(IW[16]) begin IMRS = 5'h0a; DMRS = 5'h02; RFCM[16] = 0; end
    else if(IW[10]) begin IMRS = 5'h0b; DMRS = 5'h01; RFCM[10] = 0; end
    else if(IW[ 9]) begin IMRS = 5'h10; DMRS = 5'h00; RFCM[ 9] = 0; end
    else if(IW[ 8]) begin IMRS = 5'h11; DMRS = 5'h18; RFCM[ 8] = 0; end
    else if(IW[ 7]) begin IMRS = 5'h12; DMRS = 5'h1c; RFCM[ 7] = 0; end
    else            begin IMRS = 5'h13; DMRS = 5'h1d; end
  end
  always @(RRSC or IW or LSSL or IMRS or DMRS or IRQST or IBFL or STALL or
           DRRS) begin
    RRS = 0;
    if(RRSC[0])
      RRS = IW[20:16];
    if(RRSC[1])
      RRS = IW[10:6];
    if(RRSC[2])
      RRS = {~IW[14],IW[14:11]};
    if(RRSC[3])
      RRS = IW[26] ? DMRS : IMRS;
    if(RRSC[4])
      RRS = {2'h3,RRSC[2:0]};
    if((IRQST==1) & (IBFL==0))
      RRS = 5'h1a;
    if(STALL)
      RRS = DRRS;
  end
  always @(RWS or AEXRD or DRWE or DIW or SOIR) begin
    NCRWS = RWS;
    if(AEXRD) begin
      if(DRWE[1:0] != 0)
        NCRWS[3:0] = DIW[14:11];
      if(DRWE[0])
        NCRWS[4] = DIW[15];
      if(DRWE[1])
        NCRWS[4] = ~DIW[14];
      if(DRWE[2])
        NCRWS = {2'h3,DRWE[0]?3'd5:3'd0};
    end
  if(SOIR)
    NCRWS = 5'h1e;
  end
  always @(RWES or AEXRD or DRWE or SOIR or FMWE or ALUWP or STRW) begin
    NCRWES = RWES;
    if(AEXRD) begin
      NCRWES = DRWE != 0 ? 1'b1 : 1'b0;
      if(SOIR)
        NCRWES = 1;
    end
    else if(~STRW & ~FMWE)
      NCRWES = 0;
  end
  always @(posedge CLK) begin
    if(DBGWE)
      DBGO <= DS0DI;
    if(PCCT[0])
      DBGO <= {MAIA,2'b0};
    if(AEXRD) begin
      CCOPS <= DCCOPC;
      DBGWE <= DDBGC;
      LCTDS <= DRWE[2];
      EXCDS <= DCDEX ? DIW[31:28] : 4'hf;
    end
    else begin
      if((~FMWE|ALUWP) & ~STRW) begin
        CCOPS <= 0;
        LCTDS <= 0;
      end
      DBGWE <= 0;
    end
    SRWES <= 0;
    if(NCRWES & (NCRWS[4:3]==3))
      SRWES <= {NCRWS[2:0]==1,NCRWS[2:0]==5,NCRWS[2:0]==4,NCRWS[2:0]==0};
    if(~STALL) begin
      DIW    <= IW;
      DIWS   <= IWSM;
      DIWV   <= IPH == 0;
      DACOCT <= ACOCT;
      DALUCT <= ALUCT;
      DLSCT  <= LSCT;
      DDBGC  <= DBGC;
      DLSASL <= LSASL;
      DAS0CT <= AS0CT;
      DAS1CT <= AS1CT;
      DCCOPC <= CCOPC;
      DCDEX  <= CDEX;
      DLDSTM <= LDSTM;
      DRRS   <= RRS;
      DRWE   <= RWEC;
      DRRE   <= (RRSC!=0) & (~LSCT[3]|(LSCT[2:0]>4));
      DSRWE  <= 0;
      if((RWEC==1) & (IW[15:14]==3))
        DSRWE <= {IW[13:11]==1,IW[13:11]==5,IW[13:11]==4,IW[13:11]==0};
      if(RWEC == 4)
        DSRWE[0] <= 1;
      if(RWEC == 5)
        DSRWE[2] <= 1;
      DSRLD <= 0;
      if(LSCT[3] & (LSCT[2:0]<=4) & (RRS[4:3]==3))
        DSRLD <= {RRS[2:0]==5,RRS[2:0]==4,RRS[2:0]==0};
    end
    if(~STALL & (IBFL==0)) begin
      DALUCT <= 0;
      DLSCT  <= 0;
      DAS0CT <= 2;
      DAS1CT <= 0;
      DRRE   <= 0;
      DDBGC  <= 0;
      DCDEX  <= 0;
      DLDSTM <= 0;
    end
    if((~STALL&(IBFL==0)) | FLIV)
      DIWV <= 0;
    if(STALL)
      DAS1CT[2] <= 0;
    if(~STALL)
      IPH <= NIPH;
    if(NCIBFL == 0)
      IPH <= 0;
    if(CCSCT[0])
      CCS <= CC;
    if(CCSCT[1])
      CC <= CCS;
    else
      if(CCOP[0] & EXCDT)
        CC <= CFI;
    if((IRQST==0) & (NCIRQST==1))
      if(EXRQ)
        LIRN <= 0;
      else begin
        IACK <= 1;
        LIRN <= IRN;
      end
    else
      IACK <= 0;
    if(RWE) begin
      if(RBWA == 5'h18)
        if(LCTD)
          LCTST <= LCTLD[0] ? 1'b0 : 1'b1;
        else if(FMWE)
          LCTST <= FMS1 ? 1'b0 : 1'b1;
        else
          LCTST <= LCTLD[1] ? 1'b0 : 1'b1;
      if(RBWA == 5'h19)
        CC <= DS0DI[3:0];
    end
    if(RRSC[3] & ~STALL)
      IW <= IW & RFCM;
    if(IWCS & IBFL[1]) begin
      IW   <= IB0;
      IB0  <= IB1;
      IWSM <= IBSM[0];
      IBSM <= {1'b0,IBSM[1]};
    end
    if(IIV)
      if((IBFL==0) | ((IBFL==1)&IWCS)) begin
        IW   <= II;
        IWSM <= MAST[1];
      end
      else if((IBFL==1) | IWCS) begin
        IB0     <= II;
        IBSM[0] <= MAST[1];
      end
      else begin
        IB1     <= II;
        IBSM[1] <= MAST[1];
      end
    if(LINJI) begin
      IW   <= DBGIR;
      IWSM <= CSF[0];
    end
    if(!STPCT) begin
      if(NSIF & IIV)
        PC <= MAIA;
      else if(IWCS & ~INDB)
        PC <= PC + 1'b1;
    end
    if(STPCT)
      MAIA <= PC;
    else if(SOIR)
      MAIA = DPUDI[31:2];
    else if(IIV | (NCNSIF&~NSIF) | (STPD&FLIV&~SQFFI))
      MAIA <= IA;
    if(PCCT[1])
      MAIA <= DBGIR[31:2];
    EXRQ <= (EXRQ&~SOIR) | (~EXRQ&SCVL);
    if(STPD) begin
      DBGIR <= DBGI;
      LINJI <= INJI;
    end
    INDB <= (INDB&LSTRQ) | STPD;
    STPD  <= (LSTRQ|LDBRQ) & RFST & (IRQST==0);
    LSTRQ <= STRQ & (IRQST==0) & (NCIRQST==0);
    DSOIR <= SOIR;
    CSF   <= NCCSF;
    MAST  <= NCMAST;
    IBFL  <= NCIBFL;
    NSIF  <= NCNSIF;
    RWS   <= NCRWS;
    RWES  <= NCRWES;
    IRQST <= NCIRQST;
    LDBRQ <= NCLDBRQ;
    if(~STALL)
      SHCT <= NCSHCT;
    if(IRST) begin
      IPH     <= 0;
      STPD    <= 0;
      LSTRQ   <= 0;
      LDBRQ   <= 0;
      INDB    <= 0;
      LINJI   <= 0;
      CC      <= 0;
      CSF     <= 1;
      CCS     <= 0;
      MAST    <= 0;
      NSIF    <= 1;
      DSOIR   <= 0;
      LCTST   <= 0;
      IRQST   <= 0;
      EXRQ    <= 0;
      IWSM    <= 0;
      IBSM    <= 0;
      IACK    <= 0;
      IBFL    <= 0;
      MAIA    <= {IRN,26'h0};
      DIWV    <= 0;
      DCDEX   <= 0;
      DALUCT  <= 0;
      DLSCT   <= 0;
      DSRLD   <= 0;
      DLDSTM  <= 0;
      RWES    <= 0;
      SRWES   <= 0;
      CCOPS   <= 0;
      DDBGC   <= 0;
      DBGWE   <= 0;
      EXCDS   <= 4'hf;
      LCTDS   <= 0;
      DRRE    <= 0;
      DRWE    <= 0;
      DSRWE   <= 0;
      SHCT    <= 0;
    end
    if(RST)
      RSYN <= 3;
    else if(RSYN[1])
      RSYN[1] <= 0;
    else
      RSYN <= 0;
  end
endmodule
module sf32buDpu(CLK,RST,MDI,MDRDY,RSD,RFE0,RFE1,IW,IWSM,DBGI,CC,EXRD,ALUWP,
                 S0CT,S1CT,COCT,DCCOPC,CCOP,DRRS,RBWA,RWE,LSCT,LSASL,IVFT,
                 LIRN,PC,IRQST,EXCDT,OPCT,DO,CFO,OR0,MDA,MDO,MDBS,MDWE,MDSM,
                 MBFL,MBSWE,MBDR,MADR,MAWE,MASWE,FMDR,FMWE,FMSWE,FMS1,MLST);
  input         CLK;
  input         RST;
  input  [31:0] MDI;
  input         MDRDY;
  input  [31:0] RSD;
  input  [ 1:0] RFE0;
  input  [ 1:0] RFE1;
  input  [31:0] IW;
  input         IWSM;
  input  [31:0] DBGI;
  input  [ 3:0] CC;
  input         EXRD;
  input         ALUWP;
  input  [ 2:0] S0CT;
  input  [ 2:0] S1CT;
  input  [ 3:0] COCT;
  input  [ 1:0] DCCOPC;
  input  [ 1:0] CCOP;
  input  [ 4:0] DRRS;
  input  [ 4:0] RBWA;
  input         RWE;
  input  [ 3:0] LSCT;
  input         LSASL;
  input         IVFT;
  input  [ 3:0] LIRN;
  input  [29:0] PC;
  input  [ 1:0] IRQST;
  input         EXCDT;
  input  [ 4:0] OPCT;
  output [31:0] DO;
  output [ 3:0] CFO;
  output [31:0] OR0;
  output [31:0] MDA;
  output [31:0] MDO;
  output [ 3:0] MDBS;
  output        MDWE;
  output        MDSM;
  output        MBFL;
  output [ 2:0] MBSWE;
  output [ 4:0] MBDR;
  output [ 4:0] MADR;
  output        MAWE;
  output [ 2:0] MASWE;
  output [ 4:0] FMDR;
  output        FMWE;
  output [ 2:0] FMSWE;
  output        FMS1;
  output [ 1:0] MLST;
  reg    [31:0] DO;
  reg    [ 3:0] CFO;
  reg    [31:0] OR0;
  reg    [31:0] MDA;
  reg    [31:0] MDO;
  reg    [ 3:0] MDBS;
  reg           MDWE;
  reg           MDSM;
  reg           MBFL;
  reg    [ 2:0] MBSWE;
  reg    [ 4:0] MBDR;
  reg    [ 4:0] MADR;
  reg           MAWE;
  reg    [ 2:0] MASWE;
  reg    [ 4:0] FMDR;
  reg           FMWE;
  reg    [ 2:0] FMSWE;
  reg           FMS1;
  reg    [ 1:0] MLST;
  reg    [ 4:0] OPR;
  reg    [31:0] OR1;
  reg           CRIN;
  reg    [31:0] BMSK;
  reg signed [16:0] M0S0;
  reg signed [16:0] M0S1;
  reg signed [16:0] M1S0;
  reg signed [16:0] M1S1;
  reg    [15:0] M0SH;
  reg    [ 2:0] MLOP;
  reg    [ 2:0] MBOC;
  reg    [ 1:0] MAA;
  reg    [ 3:0] MAOC;
  reg    [31:0] FMD;
  reg    [ 1:0] FMA;
  reg    [ 2:0] FMOC;
  reg    [31:0] FMDO;
  reg    [31:0] CON;
  reg    [31:0] SADD;
  reg    [31:0] SHFD;
  reg    [31:0] LBS0;
  reg    [31:0] LBTD;
  reg    [31:0] ALUD;
  reg    [ 3:0] SACC;
  reg    [ 3:0] LBCC;
  reg           MB0;
  reg           MB1;
  reg           MBA;
  reg    [ 4:0] BTIX;
  reg    [31:0] SD0;
  reg    [31:0] SD1;
  reg    [31:0] MP0;
  reg    [31:0] MP1;
  reg    [31:0] MP0O;
  reg    [31:0] MP1O;
  reg    [ 1:0] NCMLST;
  reg           NCMBFL;
  reg    [ 3:0] NCMAOC;
  reg    [31:0] NMDA;
  always @(FMWE or FMOC or FMA or FMD or FMDR) begin
    FMDO = 32'h00000000;
    if((FMOC[2:0]==4) | ((FMOC[2:1]==1)&~FMA) | ((FMOC[2:1]==0)&(FMA==0)))
      FMDO[7:0] = FMD[7:0];
    if((FMOC[2:0]==4) | ((FMOC[2:1]==1)&~FMA))
      FMDO[15:8] = FMD[15:8];
    if(FMOC[2:1] == 2'h0) begin                 
      case(FMA)
	1: FMDO[7:0] = FMD[15: 8];
	2: FMDO[7:0] = FMD[23:16];
	3: FMDO[7:0] = FMD[31:24];
	default: ;
      endcase
      if(FMDO[7] & FMOC[0])                     
	FMDO[31:8] = 24'hffffff;
    end
    if(FMOC[2:1] == 2'h1) begin                 
      if(FMA[1])
	FMDO[15:0] = FMD[31:16];
      if(FMDO[15] & FMOC[0])                    
	FMDO[31:16] = 16'hffff;
    end
    if(FMOC[2:0] == 3'h4)                       
      FMDO[31:16] = FMD[31:16];
    if(FMDR[4:2] == 7)
      FMDO[1:0] = 0;
    FMS1 = FMD[31:1] == 0;
  end
  always @(COCT or IW) begin
    CON = 0;
    if(COCT[3:2] != 0) begin
      if(COCT == 4)
        CON[31:16] = IW[31:16];
      else begin
        CON[11:0] = IW[27:16]; 
        if(COCT[3]) begin
          case(COCT[2:1])
	    0,
	    2: CON[15:12] = IW[31:28];
            1: CON[15:12] = IW[14:11];
            3: CON[15:12] = IW[ 9: 6];
          endcase
          CON[16] = COCT[2] ? IW[3] : COCT[0] & CON[15];
	end
        else if(COCT[0] & IW[27])
          CON[16:12] = 5'h1f;
      end
      if(COCT[0] & CON[16])
        CON[31:17] = 15'h7fff;
    end
    else if(COCT[0]) begin
      CON = 32'hffffffff;
      if(COCT[1])
        if(IW[5] & ~IW[3])
          CON[1:0] = 0;
        else if((IW[5:4]==1) | (IW[5:3]==7))
          CON[0] = 0;
    end
    else if(COCT[1]) begin
      CON[0] = 1;
      if(IW[5] & ~IW[3])
        CON[2:0] = 4;
      else if((IW[5:4]==1) | (IW[5:3]==7))
        CON[1:0] = 2;
    end
  end
  always @(S1CT or IW or RSD)
    BTIX = S1CT[0] ? IW[20:16] : RSD[4:0];
  always @(OR1 or OR0 or CRIN or OPR or CCOP or CC) begin
    SADD = OPR[0] ? OR1 + OR0 + CRIN : OR1 - OR0 - CRIN;
    MB0 = OR0[31];
    MB1 = OR1[31];
    MBA = SADD[31];
    if(OPR[0]) begin
      SACC[0] = (MB1&MB0) | (MB1&~MBA) | (MB0&~MBA);
      SACC[1] = ((MB1&MB0&~MBA) | (~MB1&~MB0&MBA));
    end
    else begin
      SACC[0] = (~MB1&MB0) | (~MB1&MBA)|(MB0& MBA);
      SACC[1] = ((MB1&~MB0&~MBA) | (~MB1&MB0&MBA));
    end
    SACC[2] = SADD == 0;
    SACC[3] = SADD[31];
    if(CCOP[1] & ~CC[2])
      SACC[2] = 0;
  end
  always @(OPR or OR1) begin
    SHFD = OPR[0] ? {1'b0,OR1[31:1]} : {OR1[30:0],1'b0};
    if(OPR[1:0] == 2)
      SHFD[0] = OR1[31];
    if(OPR[1:0] == 3)
      SHFD[31] = OR1[31];
  end
  always @ (OPR or OR0 or OR1 or CC or BMSK) begin
    LBS0 = OPR[2] ? BMSK : OR0;
    case(OPR[1:0])
      0: LBTD = OR1 & ~LBS0;              
      1: LBTD = OR1 &  LBS0;              
      2: LBTD = OR1 |  LBS0;              
      3: LBTD = OR1 ^  LBS0;              
    endcase
    LBCC = 4'h0;
    if(~OPR[2]) begin
      LBCC[0] =           LBTD[ 0] ^ LBTD[ 1] ^ LBTD[ 2] ^ LBTD[ 3];
      LBCC[0] = LBCC[0] ^ LBTD[ 4] ^ LBTD[ 5] ^ LBTD[ 6] ^ LBTD[ 7];
      LBCC[0] = LBCC[0] ^ LBTD[ 8] ^ LBTD[ 9] ^ LBTD[10] ^ LBTD[11];
      LBCC[0] = LBCC[0] ^ LBTD[12] ^ LBTD[13] ^ LBTD[14] ^ LBTD[15];
      LBCC[0] = LBCC[0] ^ LBTD[16] ^ LBTD[17] ^ LBTD[18] ^ LBTD[19];
      LBCC[0] = LBCC[0] ^ LBTD[20] ^ LBTD[21] ^ LBTD[22] ^ LBTD[23];
      LBCC[0] = LBCC[0] ^ LBTD[24] ^ LBTD[25] ^ LBTD[26] ^ LBTD[27];
      LBCC[0] = LBCC[0] ^ LBTD[28] ^ LBTD[29] ^ LBTD[30] ^ LBTD[31];
      LBCC[1] = CC[1]  ^ LBCC[0];
    end
    LBCC[2] = LBTD == 0;
    LBCC[3] = LBTD[31];
  end
  always @(OR0 or OR1 or OPR or SADD or SACC or SHFD or LBTD or LBCC or RWE or
           RBWA or FMWE or FMDO or ALUWP or RBWA or LSCT) begin
    ALUD  = OR0;
    CFO   = 4'h0;
    if(OPR[4:3] == 3)
      ALUD = SHFD; 
    else if(OPR[4:3] == 0)
      case(OPR[2:0])
           0,1: begin                                        
                ALUD  = SADD;
                CFO   = SACC;
              end     
           2: if(OR0[31])                                    
                ALUD = SADD;
           3: ALUD[15:0] = {OR0[7:0],OR0[15:8]};             
           6: if(OR0[31]) ALUD = 32'h00000000;               
              else if(OR0[30]) ALUD = 32'h00000001;
              else if(OR0[29]) ALUD = 32'h00000002;
              else if(OR0[28]) ALUD = 32'h00000003;
              else if(OR0[27]) ALUD = 32'h00000004;
              else if(OR0[26]) ALUD = 32'h00000005;
              else if(OR0[25]) ALUD = 32'h00000006;
              else if(OR0[24]) ALUD = 32'h00000007;
              else if(OR0[23]) ALUD = 32'h00000008;
              else if(OR0[22]) ALUD = 32'h00000009;
              else if(OR0[21]) ALUD = 32'h0000000a;
              else if(OR0[20]) ALUD = 32'h0000000b;
              else if(OR0[19]) ALUD = 32'h0000000c;
              else if(OR0[18]) ALUD = 32'h0000000d;
              else if(OR0[17]) ALUD = 32'h0000000e;
              else if(OR0[16]) ALUD = 32'h0000000f;
              else if(OR0[15]) ALUD = 32'h00000010;
              else if(OR0[14]) ALUD = 32'h00000011;
              else if(OR0[13]) ALUD = 32'h00000012;
              else if(OR0[12]) ALUD = 32'h00000013;
              else if(OR0[11]) ALUD = 32'h00000014;
              else if(OR0[10]) ALUD = 32'h00000015;
              else if(OR0[ 9]) ALUD = 32'h00000016;
              else if(OR0[ 8]) ALUD = 32'h00000017;
              else if(OR0[ 7]) ALUD = 32'h00000018;
              else if(OR0[ 6]) ALUD = 32'h00000019;
              else if(OR0[ 5]) ALUD = 32'h0000001a;
              else if(OR0[ 4]) ALUD = 32'h0000001b;
              else if(OR0[ 3]) ALUD = 32'h0000001c;
              else if(OR0[ 2]) ALUD = 32'h0000001d;
              else if(OR0[ 1]) ALUD = 32'h0000001e;
              else if(OR0[ 0]) ALUD = 32'h0000001f;
              else ALUD = 32'h00000020;
           7: ALUD = {OR0[7:0],OR0[15:8],OR0[23:16],OR0[31:24]}; 
	   default: ;
	endcase
      else begin                                           
        ALUD  = LBTD;
        CFO   = LBCC;
      end
    if(RWE & ~LSCT[3] & (RBWA[4:2]==7))                    
      ALUD[1:0] = 0;
    DO = FMWE & ~ALUWP ? FMDO : ALUD;
  end
  always @(MLST or EXRD or OPCT or MLOP)
    case(MLST)
      0: NCMLST = EXRD & (OPCT[4:3]==1);
      1: NCMLST = MLOP[2] ? 2'd0 : 2'd2;
      2: NCMLST = MLOP[1] ? 2'd3 : 2'd0;
      3: NCMLST = 0;
    endcase
  always @(RSD or RFE0 or ALUD or FMDO or MLST)
    if(RFE0[0] | MLST[1])
      SD0 = ALUD;
    else if(RFE0 == 0)
      SD0 = RSD;
    else
      SD0 = FMDO;
  always @(RFE1 or ALUD or RSD or FMDO)
    if(RFE1[0])
      SD1 = ALUD;
    else if(RFE1 == 0)
      SD1 = RSD;
    else
      SD1 = FMDO;
  always @(M0S0 or M0S1 or M1S0 or M1S1) begin
    MP0 = M0S0 * M0S1;
    MP1 = M1S0 * M1S1;
  end
  always @(MLOP or MP0 or MLST or MP1) begin
    if(MLOP[1]) begin
      MP0O = {16'h0,MP0[31:16]};
      MP1O = MLST == 2 ? MP0 : MP1;
    end
    else begin
      MP0O = MP0;
      MP1O = {MP1[15:0],16'h0};
    end
  end
  always @(MBOC or MDA or BMSK) begin
    MDO = 32'h00000000;
    if((MBOC==6) | ((MBOC==7)&~MDA[1]) | ((MBOC==5)&(MDA[1:0]==0)))
      MDO[7:0] = BMSK[7:0];
    if((MBOC==6) | ((MBOC==7)&~MDA[1]))
      MDO[15:8] = BMSK[15:8];
    if(MBOC == 3'h5)                           
      case(MDA[1:0])
	1: MDO[15: 8] = BMSK[7:0];
	2: MDO[23:16] = BMSK[7:0];
	3: MDO[31:24] = BMSK[7:0];
	default:;
      endcase
    if(MBOC == 3'h6)                           
      MDO[31:16] = BMSK[31:16];
    if((MBOC==3'h7) & MDA[1])                  
      MDO[31:16] = BMSK[15:0];
  end
   always @(LSASL or ALUD or OR1)
     NMDA = LSASL ? ALUD : OR1;
  always @(MBFL or MAOC or MDRDY or LSCT or IVFT or EXRD or EXCDT) begin
    NCMBFL = MBFL;
    if(~MAOC[3] | MDRDY)
      NCMBFL = 0;
    if((LSCT[3]|IVFT) & ~MBFL & EXRD & EXCDT)
      NCMBFL = 1;
  end
  always @(MAOC or MDRDY or MBFL or MBOC) begin
    NCMAOC = MAOC;
    if(MAOC[3] & MDRDY)
      NCMAOC[3] = 0;
    if(MBFL & (~MAOC[3]|MDRDY))
      NCMAOC = {1'b1,MBOC};
  end
  always @(posedge CLK) begin
    if(EXRD & (MLST==0)) begin
      OPR  <= OPCT;
      if(DCCOPC[1]) begin
	CRIN <= (~CCOP[0]&CC[0]) | (CCOP[0]&SACC[0]);
	if(RWE & (RBWA==5'h19) & ~FMWE)
	  CRIN <= OR0[0];
      end
      else
	CRIN <= 0;
      if(OPCT[4:3] == 1) begin
        MLOP <= OPCT[2:0];
        OPR  <= 1;
      end
    end
    if((IRQST==2) & ~MBFL)
      OPR <= 4;
    if(S0CT[0]) begin
      M0S0 <= {1'b0,SD0[15:0]};
      M1S0 <= {1'b0,SD0[15:0]};
      M0SH <= SD0[31:16];
    end
    if(S0CT[1]) begin
      M0S0 <= {(OPCT[2:0]==5)&CON[15],CON[15:0]};
      M1S0 <= {1'b0,CON[15:0]};
    end
    if(S1CT[0]) begin
      M0S1 <= {1'b0,SD1[15:0]};
      M1S1 <= {OPCT[0]&SD1[31],SD1[31:16]};
      if(S0CT[1] & ~CON[15])
        M1S1[16] <= 0;
    end
    if(MLST == 1) begin
      if(~MLOP[1])
        M1S1 <= M0S1;
      M0S0 <= {MLOP[0]&M0SH[15],M0SH};
      M1S0 <= {MLOP[0]&M0SH[15],M0SH};
    end
    if((OPCT[4:2]==5) & (~IW[25]|~S1CT[0]) & EXRD & ~MBFL) begin
      BMSK <= 0;
      case(BTIX)
        5'h00: BMSK[ 0] <= 1;
        5'h01: BMSK[ 1] <= 1;
        5'h02: BMSK[ 2] <= 1;
        5'h03: BMSK[ 3] <= 1;
        5'h04: BMSK[ 4] <= 1;
        5'h05: BMSK[ 5] <= 1;
        5'h06: BMSK[ 6] <= 1;
        5'h07: BMSK[ 7] <= 1;
        5'h08: BMSK[ 8] <= 1;
        5'h09: BMSK[ 9] <= 1;
        5'h0a: BMSK[10] <= 1;
        5'h0b: BMSK[11] <= 1;
        5'h0c: BMSK[12] <= 1;
        5'h0d: BMSK[13] <= 1;
        5'h0e: BMSK[14] <= 1;
        5'h0f: BMSK[15] <= 1;
        5'h10: BMSK[16] <= 1;
        5'h11: BMSK[17] <= 1;
        5'h12: BMSK[18] <= 1;
        5'h13: BMSK[19] <= 1;
        5'h14: BMSK[20] <= 1;
        5'h15: BMSK[21] <= 1;
        5'h16: BMSK[22] <= 1;
        5'h17: BMSK[23] <= 1;
        5'h18: BMSK[24] <= 1;
        5'h19: BMSK[25] <= 1;
        5'h1a: BMSK[26] <= 1;
        5'h1b: BMSK[27] <= 1;
        5'h1c: BMSK[28] <= 1;
        5'h1d: BMSK[29] <= 1;
        5'h1e: BMSK[30] <= 1;
        5'h1f: BMSK[31] <= 1;
      endcase
    end
    if(MAWE & MDRDY) begin
      FMWE  <= 1;
      FMSWE <= MASWE;
      FMD   <= MDI;
      FMOC  <= MAOC[2:0];
      FMDR  <= MADR;
      FMA   <= MAA;
    end
    else begin
      FMOC  <= 0;
      FMWE  <= 0;
      FMSWE <= 0;
    end
    if(MAOC[3] & MDRDY) begin
      MAWE  <= 0;
      MASWE <= 0;
    end
    if(MBFL & (~MAOC[3]|MDRDY)) begin
      MADR  <= MBDR;
      MAWE  <= ~MDWE;
      MASWE <= MBSWE;
      MAA   <= MDA[1:0];
    end
    if(~MAOC[3] | MDRDY) begin
      MDBS  <= 0;
      MDWE  <= 0;
      MBSWE <= 0;
      MDSM  <= 0;
    end
    if(IVFT & EXRD) begin
      MBDR  <= 5'h1f;
      MBOC  <= 4;
      MBSWE <= 0;
      MDSM  <= 1;
      MDBS  <= 4'hf;
      MDA   <= {RSD[31:6],LIRN,2'd0};
    end
    else if(LSCT[3] & ~MBFL & EXRD & EXCDT) begin
      BMSK  <= SD0;
      MBDR  <= DRRS;
      MDSM  <= IWSM;
      MBOC  <= LSCT[2:0];
      MDA   <= NMDA;
      MDWE  <= LSCT[2:0] > 4;
      MBSWE <= 0;
      if((LSCT[2:0]<=4) & (DRRS[4:3]==3))
        MBSWE <= {DRRS[2:0]==5,DRRS[2:0]==4,DRRS[2:0]==0};
      if((LSCT[2:1]==0) | (LSCT[2:0]==5))
        case(NMDA[1:0])
	  0: MDBS <= 4'b0001;
	  1: MDBS <= 4'b0010;
	  2: MDBS <= 4'b0100;
	  3: MDBS <= 4'b1000;
        endcase
      if((LSCT[2:1]==1) | (LSCT[2:0]==7))
        MDBS <= NMDA[1] ? 4'b1100 : 4'b0011;
      if(LSCT[2] & ~LSCT[0])
        MDBS <= 4'b1111;
      if(~EXCDT) begin
        MDBS  <= 0;
        MDWE  <= 0;
        MBSWE <= 0;
      end
    end
    if(EXRD)
      if(S0CT[0] | (MLST==2)) begin
        OR0 <= SD0;
        if((IW[27:26]==1) & IW[15] & IW[2] & IW[0]) begin 
          if((IW[5:4]==1) | (IW[5:3]==7))                 
            OR0 <= {SD0[30:0],1'b0};
          if(IW[5] & ~IW[3])                              
            OR0 <= {SD0[29:0],2'b0};
	end
      end
      else if(MLST == 3) begin
        OR0 <= {15'h0,~MLOP[0]&SACC[0],ALUD[31:16]};
        if((M1S0[16]|M1S1[16]) & ALUD[31])
          OR0[31:16] <= 16'hffff;
      end
      else if(MLST == 1)
        OR0 <= MP0O;
      else if(S0CT[1])
        OR0 <= CON;
      else if(S0CT[2])
        OR0 <= DBGI;
    if(EXRD)
      if(S1CT[0] | (~S1CT[0]&RFE1[0]))
        OR1 <= SD1;
      else if(MLST != 0)
        OR1 <= MP1O;
      else if(S1CT[1])
        OR1 <= CON;
    if(((IRQST==2)&~MBFL) | S1CT[2])
      OR1 <= {PC,2'd0};
    MLST <= NCMLST;
    MBFL <= NCMBFL;
    MAOC <= NCMAOC;
    if(RST) begin
      OPR   <= 0;
      MLST  <= 0;
      MDBS  <= 0;
      MDWE  <= 0;
      MDSM  <= 0;
      MBFL  <= 0;
      MBSWE <= 0;
      MBOC  <= 0;
      MAOC  <= 0;
      MAWE  <= 0;
      MASWE <= 0;
      FMOC  <= 0;
      FMWE  <= 0;
      FMSWE <= 0;
    end
  end
endmodule
module sf32buRf(CLK,RST,DRRS,CC,CSF,RBDO,RBDI,RBWA,RBWE,RP);
  input         CLK;
  input         RST;
  input  [ 4:0] DRRS;
  input  [ 3:0] CC;
  input  [ 2:0] CSF;
  input  [31:0] RBDO;
  input  [31:0] RBDI;
  input  [ 4:0] RBWA;
  input         RBWE;
  output [31:0] RP;
  reg    [31:0] RP;
  reg           LWDV;
  reg    [ 4:0] LWDR;
  reg    [31:0] LWDB;
  always @(LWDV or DRRS or LWDR or LWDB or RBDO or CC or CSF) begin
    RP = LWDV & (DRRS==LWDR) ? LWDB : RBDO;
    if(DRRS == 5'h1b)
      RP[1:0] = 0;
    if(DRRS == 5'h19)
      RP = {28'h0,CC};
    if(DRRS == 5'h1a)
      RP[5:0] = {3'h0,CSF};
    if(DRRS == 5'h1b)
      RP = 0;
    if(DRRS == 5'h1f)
      RP = 32'h00001414;
  end
  always @(posedge CLK) begin
    if(RBWE) begin
      LWDB <= RBDI;
      LWDR <= RBWA;
      LWDV <= 1;
    end
    if(RST)
      LWDV <= 0;
  end
endmodule
module sf32buRfb(CLK,RAD,DI,WAD,WE,DO);
  input         CLK;
  input  [ 4:0] RAD;
  input  [31:0] DI;
  input  [ 4:0] WAD;
  input         WE;
  output [31:0] DO;
  reg    [31:0] DO;
  reg    [31:0] R0;
  reg    [31:0] R1;
  reg    [31:0] R2;
  reg    [31:0] R3;
  reg    [31:0] R4;
  reg    [31:0] R5;
  reg    [31:0] R6;
  reg    [31:0] R7;
  reg    [31:0] R8;
  reg    [31:0] R9;
  reg    [31:0] RA;
  reg    [31:0] RB;
  reg    [31:0] RC;
  reg    [31:0] RD;
  reg    [31:0] RE;
  reg    [31:0] RF;
  reg    [31:0] RP;
  reg    [31:0] RQ;
  reg    [31:0] RU;
  reg    [31:0] RV;
  reg    [31:0] RW;
  reg    [31:0] RX;
  reg    [31:0] RY;
  reg    [31:0] RZ;
  reg    [31:0] LC;
  reg    [31:0] D0;
  reg    [31:0] IV;
  reg    [31:0] EP;
  reg    [31:0] TA;
  reg    [31:0] SA;
  reg    [31:0] IA;
  reg    [31:0] D1;
  always @(posedge CLK) begin
    case(RAD)
      5'h00: DO <= R0;
      5'h01: DO <= R1;
      5'h02: DO <= R2;
      5'h03: DO <= R3;
      5'h04: DO <= R4;
      5'h05: DO <= R5;
      5'h06: DO <= R6;
      5'h07: DO <= R7;
      5'h08: DO <= R8;
      5'h09: DO <= R9;
      5'h0a: DO <= RA;
      5'h0b: DO <= RB;
      5'h0c: DO <= RC;
      5'h0d: DO <= RD;
      5'h0e: DO <= RE;
      5'h0f: DO <= RF;
      5'h10: DO <= RP;
      5'h11: DO <= RQ;
      5'h12: DO <= RU;
      5'h13: DO <= RV;
      5'h14: DO <= RW;
      5'h15: DO <= RX;
      5'h16: DO <= RY;
      5'h17: DO <= RZ;
      5'h18: DO <= LC;
      5'h19: DO <= D0;
      5'h1a: DO <= IV;
      5'h1b: DO <= EP;
      5'h1c: DO <= TA;
      5'h1d: DO <= SA;
      5'h1e: DO <= IA;
      5'h1f: DO <= D1;
    endcase
    if(WE)
      case(WAD)
        5'h00: R0 <= DI;
        5'h01: R1 <= DI;
        5'h02: R2 <= DI;
        5'h03: R3 <= DI;
        5'h04: R4 <= DI;
        5'h05: R5 <= DI;
        5'h06: R6 <= DI;
        5'h07: R7 <= DI;
        5'h08: R8 <= DI;
        5'h09: R9 <= DI;
        5'h0a: RA <= DI;
        5'h0b: RB <= DI;
        5'h0c: RC <= DI;
        5'h0d: RD <= DI;
        5'h0e: RE <= DI;
        5'h0f: RF <= DI;
        5'h10: RP <= DI;
        5'h11: RQ <= DI;
        5'h12: RU <= DI;
        5'h13: RV <= DI;
        5'h14: RW <= DI;
        5'h15: RX <= DI;
        5'h16: RY <= DI;
        5'h17: RZ <= DI;
        5'h18: LC <= DI;
        5'h19: D0 <= DI;
        5'h1a: IV <= DI;
        5'h1b: EP <= DI;
        5'h1c: TA <= DI;
        5'h1d: SA <= DI;
        5'h1e: IA <= DI;
        5'h1f: D1 <= DI;
        default: ;
      endcase
  end
endmodule
